/*
 * Copyright (C) 2017 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.android.car.settings.system;

import android.car.drivingstate.CarUxRestrictions;
import android.content.Context;
import android.os.Build;

import androidx.annotation.VisibleForTesting;
import androidx.preference.Preference;

import com.android.car.settings.R;
import com.android.car.settings.common.FragmentController;
import com.android.car.settings.common.Logger;
import com.android.car.settings.common.PreferenceController;
import com.android.settingslib.DeviceInfoUtils;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.FutureTask;

/** Updates the hardware info entry summary with the device model. */
public class HardwareInfoPreferenceController extends PreferenceController<Preference> {

    private static final Logger LOG = new Logger(HardwareInfoPreferenceController.class);

    public HardwareInfoPreferenceController(Context context, String preferenceKey,
            FragmentController fragmentController, CarUxRestrictions uxRestrictions) {
        super(context, preferenceKey, fragmentController, uxRestrictions);
    }

    @Override
    protected Class<Preference> getPreferenceType() {
        return Preference.class;
    }

    @Override
    protected void updateState(Preference preference) {
        preference.setSummary(
                getContext().getString(R.string.hardware_info_summary, getDeviceModel()));
    }

    private String getDeviceModel() {
        FutureTask<String> msvSuffixTask = createMsvSuffixTask();

        msvSuffixTask.run();
        try {
            // Wait for msv suffix value.
            String msvSuffix = msvSuffixTask.get();
            return Build.MODEL + msvSuffix;
        } catch (ExecutionException e) {
            LOG.e("Execution error, so we only show model name");
        } catch (InterruptedException e) {
            LOG.e("Interruption error, so we only show model name");
        }
        // If we can't get an msv suffix value successfully,
        // it's better to return model name.
        return Build.MODEL;
    }

    @VisibleForTesting
    FutureTask<String> createMsvSuffixTask() {
        return new FutureTask<>(() -> DeviceInfoUtils.getMsvSuffix());
    }
}
